/* This is version 0. of slider by Pavle Michko at Euphoria,  june 2013. 
 * */
#include <ncurses.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "frame.h"
#include "page.h"
#include "parse.tab.h"
#include "signal.h"
#include <locale.h>

typedef struct {
    frame ptr;
    int vis;
    PANEL *pan;
} panstate;
#define MAXPAN 32

panstate PS[MAXPAN];

static void finish(int sig)
{
    endwin();

    /* do your non-curses wrapup here */

    exit(0);
}

int row, col, nblines = 25, nbcols = 80;
int startx, starty;

int count = 0;

page ss;
frame last;


void shell(void)
{
    def_prog_mode();		/* save current tty modes */
    endwin();			/* restore original tty modes */
    system("sh");		/* run shell */
    refresh();			/* restore save modes, repaint screen */
}

char *fname = NULL;


void play(frame z)
{
    char cmd[1024];
    if (!z->prog)
	return;
    switch (z->prog) {
    case VIM:
	sprintf(cmd, "vim  %s", z->arg);
	break;
    case LYNX:
	sprintf(cmd, "linx %s", z->arg);
	break;
    case SHELL:
	sprintf(cmd, "bash -vc \"clear;%s;read;\" ", z->arg);
	break;
    }
    def_prog_mode();
    endwin();
    system(cmd);
    refresh();
}

void apropos(frame z)
{
    char cmd[128];

    sprintf(cmd, "linx http://fr.wikipedia.org/wiki/%s", z->key);
    def_prog_mode();
    endwin();
    system(cmd);
    refresh();
}

void manual(frame z)
{
    char cmd[128];
    sprintf(cmd, "man %s", z->key);
    def_prog_mode();
    endwin();
    system(cmd);
    refresh();
}

void helpbash(frame z)
{
    char cmd[128];

    sprintf(cmd, "help %s", z->key);
    def_prog_mode();
    endwin();
    system(cmd);
    refresh();
}

int option(int argc, char *argv[])
{
    int opt;
    while ((opt = getopt(argc, argv, "l:c:")) != -1) {
	switch (opt) {
	case 'l':
	    nblines = atoi(optarg);
	    break;
	case 'c':
	    nbcols = atoi(optarg);
	    break;
	default:		/* '?' */
	    fprintf(stderr, "Usage: %s [-lNB -cNB]\n", argv[0]);
	    exit(EXIT_FAILURE);
	}
    }


    if (optind != argc - 1) {
	fprintf(stderr, "input filename expected\n");
	exit(EXIT_FAILURE);
    }

    fname = strdup(argv[optind]);

    return 1;
}

void help(void)
{
    int x = 3;
    PANEL *pan;
    WINDOW *win;
    win = newwin(12, 60, 1, 1);
    labelw(win, "help", 4);
    mvwprintw(win, x++, 3, "change slide   : up-down page");
    mvwprintw(win, x++, 3, "change frame   : up-down arrow");
    mvwprintw(win, x++, 3, "change keyword : right-left arrow");
    mvwprintw(win, x++, 3, "open/close     : ' ' x");
    mvwprintw(win, x++, 3, "play frame     : l");
    mvwprintw(win, x++, 3, "man keyword    : m");
    mvwprintw(win, x++, 3, "wiki keyword   : w");
    mvwprintw(win, x++, 3, "shell  : s");

    pan = new_panel(win);
    update_panels();
    doupdate();
    getch();
    del_panel(pan);
    delwin(win);
}

char *PROGNAME[5] = { NULL, "vim", "lynx", "shell", NULL };

void info(frame z)
{
    int x = 3;
    PANEL *pan;
    WINDOW *win;
    win = newwin(10, col - 5, 1, 1);
    labelw(win, "info", 4);
    mvwprintw(win, x++, 3, "lab=%s (%d)", z->label, z->hidden);
    wprintw(win, " at %d x %d w=%d l=%d", z->x, z->y, z->width, z->height);
    if (z->prog)
	mvwprintw(win, x++, 3, "play=%s %-50s ", PROGNAME[z->prog],
		  z->arg);

    pan = new_panel(win);
    update_panels();
    doupdate();
    getch();
    del_panel(pan);
    delwin(win);
}


void division(int *nbl, int *nbc, char *s)
{
    *nbl = 1;
    *nbc = 0;
    if (!s)
	return;
    while (*s) {
	if (*s++ == '!')
	    (*nbl)++;
	else
	    (*nbc)++;
    }
    *nbc = *nbc / *nbl;
}

void xyz(frame z, int vs, int hs, char *s)
{
    int l = 0, c = 0;
    if (!s)
	return;
    int x = 0, y = 0, w = 0, h = 0;

    while (*s && *s != *(z->plan)) {
	if (*s++ == '!') {
	    l++;
	    c = 0;
	} else
	    c++;
    }
    if (!*s)
	return;
    x = l;
    y = c;
    while (*s) {
	if (*s == *(z->plan)) {
	    h = l;
	    w = c;
	}
	if (*s == '!') {
	    l++;
	    c = 0;
	} else
	    c++;
	s++;
    }
    h = (h - x + 1) * vs;
    w = (w - y + 1) * hs;

    if (z->height == 0 || z->height > h)
	z->height = h;
    if (z->width == 0 || z->width > w)
	z->width = w;
    x = x * vs;
    y = y * hs;
    z->x += x + (h - z->height) / 2;
    z->y += y + (w - z->width) / 2;
}

void prepare(page list)
{
    int vspace, hspace, vmargin = 1, hmargin = 1;

    page head = list;
    frame z;

    do {
	list->num = count++;
	list->slide = newwin(list->height, list->width, list->x, list->y);
	wattron(list->slide, COLOR_PAIR(1));
	if (list->title) {
	    labels(list);
	    vmargin = 3;
	} else
	    vmargin = 1;
	z = list->fil;
	list->count = 0;
	list->active = NULL;
	division(&vspace, &hspace, list->plan);
	if (vspace)
	    vspace = (list->height - vmargin - 1) / vspace;
	if (hspace)
	    hspace = (list->width - 2 * hmargin) / hspace;
	while (z) {
	    if (z->plan)
		xyz(z, vspace, hspace, list->plan);
	    z->x += vmargin;
	    z->y += hmargin;
	    z->win = newwin(z->height, z->width, z->x, z->y);
	    z->pos = 0;
	    label(z);
	    switch (z->type) {
	    case ZONE:
	    case ALARM:
		text(z);
		break;
	    case ASCII:
		ascii(z);
		break;
	    case TERM:
	    case THIS:
		term(z);
		break;
	    }
	    z = z->next;
	    list->count++;
	}
	list = list->next;

    } while (list != head);
}

int mkpanels(page ss)
{
    frame z;
    int index = 0;
    PS[index].pan = new_panel(ss->slide);
    PS[index].vis = 1;
    index++;
    z = ss->fil;
    while (z) {
	PS[index].ptr = z;
	PS[index].pan = new_panel(z->win);
	PS[index].vis = (0 == z->hidden);
	if (!PS[index].vis)
	    hide_panel(PS[index].pan);
	index++;
	z = z->next;
    }
    return index;
}

void init(void)
{
    (void) signal(SIGINT, finish);
    setlocale(LC_ALL, "");
    initscr();
    start_color();

    init_pair(RED_PAIR, COLOR_RED, COLOR_BLACK);
    init_pair(GREEN_PAIR, COLOR_GREEN, COLOR_BLACK);
    init_pair(WHITE_PAIR, COLOR_WHITE, COLOR_BLACK);
    init_pair(BLUE_PAIR, COLOR_BLUE, COLOR_BLACK);
    init_pair(YELLOW_PAIR, COLOR_YELLOW, COLOR_BLACK);
    init_pair(CYAN_PAIR, COLOR_CYAN, COLOR_BLACK);
    init_pair(MAGENTA_PAIR, COLOR_MAGENTA, COLOR_BLACK);

    getmaxyx(stdscr, row, col);

    keypad(stdscr, true);
    noecho();
}

int main(int argc, char *argv[])
{
    int car;

    int index, maxpan, active;
    WINDOW *win;
    int x, y, X, Y;

    page old;

    option(argc, argv);
    init();

    ss = loadpage(fname);
    prepare(ss);
    old = NULL;
    do {
	if (old != ss) {
	    if (old)
		for (index = 0; index < maxpan; index++)
		    del_panel(PS[index].pan);
	    maxpan = mkpanels(ss);
	    for (index = 0; index < maxpan; index++)
		if (PS[index].vis)
		    active = index;
	}
	old = ss;
	wattron(ss->slide, COLOR_PAIR(2));
	mvwprintw(ss->slide, ss->height - 1, 2, "%2d/%2d", 1 + active,
		  maxpan);
	mvwprintw(ss->slide, ss->height - 1, ss->width - 10, "%2d/%2d",
		  1 + ss->num, count);

	update_panels();
	doupdate();

	car = getch();
	switch (car) {
	case KEY_PPAGE:
	    ss = ss->prev;
	    break;
	case ' ':
	    index = 0;
	    while (index < maxpan && PS[index].vis)
		index++;
	    if (index < maxpan) {
		show_panel(PS[index].pan);
		PS[index].vis = 1;
		PS[index].ptr->hidden = 0;
		top_panel(PS[index].pan);
		active = index;
		break;
	    }
	case KEY_NPAGE:
	    ss = ss->next;
	    break;
	case 'u':
	    if (active == 0)
		break;
	    win = panel_window(PS[active].pan);
	    getbegyx(win, x, y);
	    if (x > 0)
		move_panel(PS[active].pan, x - 1, y);
	    break;
	case 'd':
	    if (active == 0)
		break;
	    win = panel_window(PS[active].pan);
	    getbegyx(win, x, y);
	    getmaxyx(win, X, Y);
	    if (X < row - 1)
		move_panel(PS[active].pan, x + 1, y);
	    break;
	case 'l':
	case '\n':
	    if (active == 0)
		break;
	    play(PS[active].ptr);
	    break;
	case 'r':
	    if (active == 0)
		break;
	    win = panel_window(PS[active].pan);
	    getbegyx(win, x, y);
	    getmaxyx(win, X, Y);
	    if (Y < col - 1)
		move_panel(PS[active].pan, x, y + 1);
	    break;
	    break;
	case KEY_UP:
	    index = active - 1;
	    for (; index > 0 && !PS[index].vis; index--);
	    if (index > 0) {
		active = index;
		top_panel(PS[active].pan);
	    }
	    break;
	case KEY_RIGHT:
	    if (PS[active].ptr->pos == PS[active].ptr->lim)
		PS[active].ptr->pos = -1;
	    PS[active].ptr->pos++;
	    text(PS[active].ptr);
	    break;
	case KEY_LEFT:
	    if (!PS[active].ptr->pos)
		break;
	    PS[active].ptr->pos--;
	    text(PS[active].ptr);
	    break;
	case KEY_DOWN:
	    index = active + 1;
	    for (; index < maxpan && !PS[index].vis; index++);
	    if (index < maxpan) {
		active = index;
		top_panel(PS[active].pan);
	    }
	    break;
	case 's':
	    shell();
	    break;
	case 'g':
	    system("vi parse.y &");
	    break;
	case 'h':
	    help();
	    break;
	case 'b':
	    helpbash(PS[active].ptr);
	    break;
	case 'i':
	    info(PS[active].ptr);
	    break;
	case 'w':
	    apropos(PS[active].ptr);
	    break;
	case 'm':
	    manual(PS[active].ptr);
	    break;
	case 'x':
	    index = maxpan - 1;
	    while (index > 0 && !PS[index].vis)
		index--;
	    if (index > 0) {
		hide_panel(PS[index].pan);
		PS[index].vis = 0;
		PS[index].ptr->hidden = 1;
		if (index == active)
		    active = 0;
	    }
	}

    }
    while (car != 'q');

    endwin();
    return 0;
}
